from bs4 import BeautifulSoup
import requests
import json
from wowhead_maps import slotDict, armorDict, specDict


def get_items(url):
    r = requests.get(url)
    soup = BeautifulSoup(r.content, "html.parser")
    soup = soup.prettify()

    # scraping stuff to get the items
    ind = soup.find("{template: 'item', id: 'drops'")

    soup = soup[ind:]
    ind = soup.find("data:[")
    soup = soup[ind + 6 :]

    ind2 = soup.find("new Listview({template: 'npc'")
    ind2 = ind2 - 5
    soup = "[" + soup[:ind2] + "]"

    return json.loads(soup)


def parse_items(items):
    wearables = {}
    for item in items:
        # make sure it's a wearable item and not a legendary
        if item["slot"] > 0 and item["quality"] != 5:
            if item["slot"] not in slotDict.keys():
                print(item["name"], item["slot"])
            else:
                id = item["id"]
                wearables[id] = {}
                wearables[id]["name"] = item["name"]
                wearables[id]["slot_name"] = slotDict[item["slot"]]

                # save specs for item
                if "specs" in item.keys():
                    wearables[id]["specs"] = []
                    for spec in item["specs"]:
                        try:
                            wearables[id]["specs"].append(specDict[spec])
                        except:
                            pass

                # get armor type
                if item["subclass"] in armorDict.keys():
                    wearables[id]["armor_type"] = armorDict[item["subclass"]]
                else:
                    wearables[id]["armor_type"] = "None"

    return wearables


def format_items(items):
    formattedItems = {}
    for dungeon in items.keys():
        dungeonItems = items[dungeon]

        # create class/spec first dictionary
        for key, value in specDict.items():
            if value["class"] not in formattedItems.keys():
                formattedItems[value["class"]] = {}

            if value["spec"] not in formattedItems[value["class"]].keys():
                formattedItems[value["class"]][value["spec"]] = {}

        # put items where they belong
        itemNames = set()
        for itemNumber in dungeonItems.keys():
            item = dungeonItems[itemNumber]

            if item["name"] not in itemNames:
                if "specs" in item.keys():
                    for spec in item["specs"]:
                        # items, inner most layer
                        formattedItems[spec["class"]][spec["spec"]][itemNumber] = {
                            "dungeon": dungeon,
                            "slot": item["slot_name"],
                            "armor": item["armor_type"],
                        }
                else:
                    for key, value in formattedItems.items():
                        for k, v in formattedItems[key].items():
                            formattedItems[key][k][itemNumber] = {
                                "dungeon": dungeon,
                                "slot": item["slot_name"],
                                "armor": item["armor_type"],
                            }

            # tracking to make sure there are not duplicate items
            itemNames.add(item["name"])

    return formattedItems


if __name__ == "__main__":
    # dragon flight dungeons
    dungeons = {
        "Ruby Life Pools": "https://www.wowhead.com/zone=14063/ruby-life-pools",
        "The Nokhud Offensive": "https://www.wowhead.com/zone=13982/the-nokhud-offensive",
        "The Azure Vault": "https://www.wowhead.com/zone=13954/the-azure-vault",
        "Algeth'ar Academy": "https://www.wowhead.com/zone=14032/algethar-academy",
        "Halls of Valor": "https://www.wowhead.com/zone=7672/halls-of-valor",
        "Court of Stars": "https://www.wowhead.com/zone=8079/court-of-stars",
        "Shadowmoon Burial Grounds": "https://www.wowhead.com/zone=6932/shadowmoon-burial-grounds",
        "Temple of the Jade Serpent": "https://www.wowhead.com/zone=5956/temple-of-the-jade-serpent",
    }

    # get the items from scraping the wowhead page
    items = {}
    for dungeon, url in dungeons.items():
        items[dungeon] = {}
        items[dungeon] = get_items(url)
        items[dungeon] = parse_items(items[dungeon])

    # change the json layering
    items = format_items(items)

    # write out json
    with open("items.json", "w") as f:
        json.dump(items, f, indent=4)

    with open("lua_formatted_items.txt", "w") as f:
        f.write("local dungeonItems = {\n")
        for classes in items:
            f.write('\t[L["' + classes + '"]] = {\n')
            for specs in items[classes]:
                f.write('\t\t[L["' + specs + '"]] = {\n')
                for item in items[classes][specs]:
                    f.write("\t\t\t[" + str(item) + "] = {\n")
                    # item info
                    f.write(
                        '\t\t\t\t["armor"] = "'
                        + items[classes][specs][item]["armor"]
                        + '",\n'
                    )
                    f.write(
                        '\t\t\t\t["slot"] = "'
                        + items[classes][specs][item]["slot"]
                        + '",\n'
                    )
                    f.write(
                        '\t\t\t\t["dungeon"] = "'
                        + items[classes][specs][item]["dungeon"]
                        + '",\n'
                    )

                    f.write("\t\t\t},\n")
                f.write("\t\t},\n")
            f.write("\t},\n")
        f.write("}")
